"""
palaver definition: an unnecessarily elaborate or complex procedure
(A.K.A. CASA in a nutshell - fight me)
"""
import argparse
import logging
import re
import subprocess

import glob

import json

from casa_envoy.auditor import AuditDirectories, AuditFiles
from casa_envoy.interfaces import CasaLauncherIF
from workspaces.system.schema import AbstractFile
from pycapo import CapoConfig
import sys
import os


"""
Launch CASA processing jobs via Workspaces
"""

logger = logging.getLogger("casa_envoy")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class CalibrationLauncher(CasaLauncherIF):

    def __init__(self, ppr: AbstractFile, metadata: AbstractFile):
        self.ppr = ppr
        self.metadata = metadata

    def run(self):
        logger.info("RUNNING CASA!")
        subprocess.Popen("PYTHONPATH='' xvfb-run -e ${initialdir}/xvfb-run.err.txt -d -s \"-screen 0 800x600x16\" "
                         "${CASA_HOME}/bin/casa --pipeline --nogui --nologger -c "
                         "${CASA_HOME}/pipeline/pipeline/runvlapipeline.py ${PPR_FILENAME} || true",
                         shell=True, executable="/bin/bash")

    def setup_environment(self, parameters: dict):
        os.environ['SCIPIPE_ROOTDIR'] = parameters["rootDirectory"]
        os.environ['CASA_HOME'] = parameters["homeForReprocessing"]
        os.environ['PPR_FILENAME'] = str(self.ppr)

        logger.info("Checking processing environment:")
        env1 = os.environ.get('SCIPIPE_ROOTDIR')
        logger.info(f"SCIPIPE_ROOTDIR: {env1}")
        env2 = os.environ.get('CASA_HOME')
        logger.info(f"CASA_HOME: {env2}")
        env3 = os.environ.get('PPR_FILENAME')
        logger.info(f"PPR_FILENAME: {env3}")
        if 'None' in [env1, env2, env3]:
            logger.info("Environment setup Failed!")
        else:
            logger.info("Environment ready for processing")

    def check_logs(self):
        logger.info("CHECKING CASA LOGS!")

        casa_logs = glob.glob('casa-*.log')

        for file in casa_logs:
            if re.match("^.*SEVERE\sflagmanager.*$", open(file).read()):
                logger.info("CASA ERROR!")
            else:
                logger.info("CASA Success!")


def _get_settings():
    use_casa = CapoConfig().getboolean("edu.nrao.archive.workspaces.ProcessingSettings.useCasa")
    root_dir = CapoConfig().settings("edu.nrao.archive.workspaces.ProcessingSettings").rootDirectory
    casa_home = CapoConfig().settings("edu.nrao.archive.workflow.config.CasaVersions").homeForReprocessing

    return {"useCasa": use_casa,"homeForReprocessing": casa_home, "rootDirectory": root_dir}


def parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces CASA processing launcher",
        formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        "--standard-cal",
        nargs=2,
        action="store",
        required=False,
        help="run the standard calibration CASA pipeline"
    )
    return parser


def check_calibratable(metadata_filename) -> bool:
    with open(str(metadata_filename)) as json_meta:
        metadata = json.loads(json_meta.read())
        spl = metadata['productLocator']
        if "execblock" in spl:
            return True
        else:
            logger.info("SPL ERROR: This product locator is not calibratable!")
            return False


def run_audit(ppr: str, metadata: str):
    dir_audit = AuditDirectories(ppr).audit()
    if dir_audit:
        logger.info("Directory audit successful!")
    else:
        logger.info("FAILURE: directory structure audit was unsuccessful!")

    audit = AuditFiles([ppr, metadata]).audit()
    if audit:
        logger.info("File audit successful!")
    else:
        logger.info("FAILURE: file audit was unsuccessful!")


def main():
    args = parser().parse_args()
    metadata = args.standard_cal[0]
    ppr = args.standard_cal[1]

    run_audit(ppr, metadata)

    if check_calibratable(metadata):
        settings = _get_settings()
        if args.standard_cal:
            if settings.get("useCasa"):
                launcher = CalibrationLauncher(metadata, ppr)
                launcher.setup_environment(settings)
                launcher.run()
                launcher.check_logs()
            else:
                logger.info("RUNNING VELA!")
                subprocess.Popen(["vela", "--standard-cal", metadata, ppr])
    else:
        logger.info("TYPE ERROR: Provided SPL is not type execution block!")
