"""
Forger has two tasks: faking CASA logs and faking CASA products
"""
import logging
import os

import json
import tarfile
import pendulum.date
import vela.vela.content.forger_content as content


class VelaLog:
    def __init__(self, metadata: str):
        self.logger = logging.getLogger("vela")
        self.metadata = metadata

    def forge_logs(self, parameters: dict):
        # forging three files. 2 for pipeline completion, 1 log file
        # vela-pipeline.sh.out.txt
        # vela-pipeline.sh.err.txt
        # vela-<date_as_string>-<timestring>.log

        path = parameters["destinationDirectory"] + "/working"
        os.chdir(path)

        # make log files
        pipe_out = open("vela-pipeline.sh.out.txt", "x")
        pipe_out.write(forged_content())
        pipe_out.close()

        pipe_err = open("vela-pipeline.sh.err.txt", "x")
        pipe_err.write(forged_content())
        pipe_err.close()

        datestring = pendulum.now().format('YYYYMMDD')
        timestring = pendulum.now().utcnow().format("hhmmss")

        log = open(f"vela-{datestring}-{timestring}.log", "w")
        log.write(forged_content())
        log.close()

        self.logger.info("Logs forging complete.")


class VelaProduct:
    def __init__(self, metadata: str):
        self.logger = logging.getLogger("vela")
        self.metadata = metadata

    def forge_cal_products(self, parameters: dict):
        # forge products so there is something to test delivery with

        path = parameters["destinationDirectory"] + "/products"
        os.chdir(path)

        sdm_id = parameters["fileSetIds"]
        filename = sdm_id + ".ms.calapply.txt"

        calapply = open(filename, "x")
        calapply.write("I am a calibration file.")
        calapply.close()

        self.forge_weblog(path)

        self.logger.info("Forging products complete.")

    def forge_weblog(self, path):
        self.logger.info("Forging weblog...")

        datestring = pendulum.now().format('YYYYMMDD')
        timestring = pendulum.now().utcnow().format('hhmmss')
        dirname = f"pipeline-{datestring}-{timestring}"
        os.mkdir(dirname)
        os.chdir(path + "/" + dirname)
        os.mkdir("html")
        os.chdir(path + "/" + dirname + "/html")

        file1 = open(f"vela-{datestring}-{timestring}.log", "x")
        file1.write("I am a weblog file")
        file1.close()

        file2 = open("index.html", "x")
        file2.write("I am a weblog file")
        file2.close()

        file3 = open("qa_notes.html", "x")
        file3.write("I am QA Notes")
        file3.close()

        os.chdir("../../")

        with tarfile.open("weblog.tgz", "w:gz") as tar:
            tar.add(path + "/" + dirname, arcname=os.path.basename(dirname))
            tar.close()

        self.logger.info("Weblog forging complete.")


def read_metadata(filename: str) -> dict:
    with open(filename) as file:
        return json.loads(file.read())


def forged_content() -> str:
    return content.file_content()


# write fake logs and products
def forge(metadata: str):
    parameters = read_metadata(metadata)
    fake_logs = VelaLog(metadata)
    fake_logs.forge_logs(parameters)

    fake_products = VelaProduct(metadata)
    fake_products.forge_cal_products(parameters)
